@@ -12,13 +12,11 @@ from typing import (
     Iterable,
     Optional,
     Sequence,
-    Tuple,
     Type,
 )
 from refinery.lib.crypto import (
     CipherObjectFactory,
     CipherInterface,
-    SpecifiedAtRuntime,
 )
 from refinery.lib.argformats import (
     Option,
@@ -34,27 +32,10 @@ from refinery.units import (
 )
 
 
-class CipherExecutable(Executable):
-    """
-    A metaclass for the abstract class `refinery.units.crypto.cipher.CipherUnit` which
-    normalizes the class variable `key_sizes` containing an iterable of all possible
-    key sizes that are acceptable for the represented cipher.
-    """
+class CipherUnit(Unit, abstract=True):
 
-    def __new__(mcs, name, bases: tuple, nmspc: dict, abstract=False, blocksize=1, key_sizes=None):
-        nmspc.setdefault('blocksize', blocksize)
-        nmspc.setdefault('key_sizes', key_sizes)
-        return super(CipherExecutable, mcs).__new__(mcs, name, bases, nmspc, abstract=abstract)
-
-    def __init__(cls, name, bases, nmspc, abstract=False, **_):
-        cls.key_sizes = (cls.key_sizes,) if isinstance(cls.key_sizes, int) else tuple(cls.key_sizes or ())
-        super(CipherExecutable, cls).__init__(name, bases, nmspc, abstract=abstract)
-
-
-class CipherUnit(Unit, metaclass=CipherExecutable, abstract=True):
-
-    key_sizes: ClassVar[Sequence[int]]
-    blocksize: ClassVar[int]
+    key_size: Optional[Sequence[int]] = None
+    block_size: int
 
     def __init__(self, key: Arg(help='The encryption key.'), **keywords):
         super().__init__(key=key, **keywords)
@@ -68,9 +49,9 @@ class CipherUnit(Unit, metaclass=CipherExecutable, abstract=True):
         raise NotImplementedError
 
     def process(self, data: ByteString) -> ByteString:
-        if self.key_sizes and len(self.args.key) not in self.key_sizes:
+        if self.key_size and len(self.args.key) not in self.key_size:
             import itertools
-            key_size_iter = iter(self.key_sizes)
+            key_size_iter = iter(self.key_size)
             key_size_options = [str(k) for k in itertools.islice(key_size_iter, 0, 5)]
             try:
                 next(key_size_iter)
@@ -92,6 +73,8 @@ class CipherUnit(Unit, metaclass=CipherExecutable, abstract=True):
 
 class StreamCipherUnit(CipherUnit, abstract=True):
 
+    block_size = 1
+
     def __init__(
         self, key,
         stateful: Arg.Switch('-s', help='Do not reset the key stream while processing the chunks of one frame.') = False,
@@ -132,16 +115,16 @@ class StreamCipherUnit(CipherUnit, abstract=True):
     decrypt = encrypt
 
 
-_PADDINGS_LIB = ['pkcs7', 'iso7816', 'x923']
-_PADDING_NONE = 'raw'
-_PADDINGS_ALL = _PADDINGS_LIB + [_PADDING_NONE]
+PADDINGS_LIB = ['pkcs7', 'iso7816', 'x923']
+PADDING_NONE = 'raw'
+PADDINGS_ALL = PADDINGS_LIB + [PADDING_NONE]
 
 
 class BlockCipherUnitBase(CipherUnit, abstract=True):
     def __init__(
         self, key, iv: Arg('-i', '--iv', help=(
-            'Specifies the initialization vector. If none is specified, then a block of zero bytes is used.')) = B'',
-        padding: Arg.Choice('-p', type=str.lower, choices=_PADDINGS_ALL, metavar='P', help=(
+            'Specifies the initialization vector. If none is specified, then a block of zero bytes is used.')) = None,
+        padding: Arg.Choice('-p', type=str.lower, choices=PADDINGS_ALL, metavar='P', help=(
             'Choose a padding algorithm ({choices}). The raw algorithm does nothing. By default, all other algorithms '
             'are attempted. In most cases, the data was not correctly decrypted if none of these work.')
         ) = None,
@@ -149,24 +132,28 @@ class BlockCipherUnitBase(CipherUnit, abstract=True):
         **keywords
     ):
         if not padding and raw:
-            padding = _PADDING_NONE
-        iv = iv or bytes(self.blocksize)
+            padding = PADDING_NONE
         super().__init__(key=key, iv=iv, padding=padding, **keywords)
 
+    @property
+    @abc.abstractmethod
+    def block_size(self) -> int:
+        raise NotImplementedError
+
     @property
     def iv(self) -> ByteString:
-        return self.args.iv
+        return self.args.iv or bytes(self.block_size)
 
     def _default_padding(self) -> Optional[str]:
-        return _PADDINGS_LIB[0]
+        return self.args.padding
 
     def reverse(self, data: ByteString) -> ByteString:
         padding = self._default_padding()
         if padding is not None:
             self.log_info('padding method:', padding)
-            if padding in _PADDINGS_LIB:
+            if padding in PADDINGS_LIB:
                 from Crypto.Util.Padding import pad
-                data = pad(data, self.blocksize, padding)
+                data = pad(data, self.block_size, padding)
         return super().reverse(data)
 
     def process(self, data: ByteString) -> ByteString:
@@ -176,14 +163,13 @@ class BlockCipherUnitBase(CipherUnit, abstract=True):
             return result
 
         from Crypto.Util.Padding import unpad
-        padding = [padding, *(p for p in _PADDINGS_LIB if p != padding)]
-        self.log_debug(padding)
+        padding = [padding, *(p for p in PADDINGS_LIB if p != padding)]
 
         for p in padding:
-            if p == _PADDING_NONE:
+            if p == PADDING_NONE:
                 return result
             try:
-                unpadded = unpad(result, self.blocksize, p.lower())
+                unpadded = unpad(result, self.block_size, p.lower())
             except Exception:
                 pass
             else:
@@ -194,36 +180,40 @@ class BlockCipherUnitBase(CipherUnit, abstract=True):
             partial=result)
 
 
-class StandardCipherExecutable(CipherExecutable):
+class StandardCipherExecutable(Executable):
 
     _available_block_cipher_modes: ClassVar[Type[Option]]
-    _cipher_object_factory: ClassVar[CipherObjectFactory]
+    _cipher_factory: ClassVar[Optional[CipherObjectFactory]]
 
     def __new__(mcs, name, bases, nmspc, cipher: Optional[CipherObjectFactory] = None):
-        keywords = dict(abstract=not cipher)
-        if cipher and cipher is not SpecifiedAtRuntime:
-            keywords.update(blocksize=cipher.block_size)
-            keywords.update(key_sizes=cipher.key_size)
+        keywords = dict(abstract=(cipher is None))
         return super(StandardCipherExecutable, mcs).__new__(mcs, name, bases, nmspc, **keywords)
 
-    def __init__(cls, name, bases, nmspc, cipher: Optional[CipherObjectFactory] = None):
+    def __init__(_class, name, bases, nmspc, cipher: Optional[CipherObjectFactory] = None):
         abstract = cipher is None
-        super(StandardCipherExecutable, cls).__init__(name, bases, nmspc, abstract=abstract)
-        cls._cipher_object_factory = cipher
-        try:
-            block_size = cipher.block_size
-        except AttributeError:
-            pass
+        super(StandardCipherExecutable, _class).__init__(name, bases, nmspc, abstract=abstract)
+        _class._cipher_factory = cipher
+        if abstract:
+            return
+        b_size = cipher.block_size
+        k_size = cipher.key_size
+        if b_size is not None:
+            _class.block_size = b_size
         else:
-            if block_size <= 1:
-                return
-        if abstract or 'mode' not in cls._argument_specification:
+            b_size = getattr(_class, 'block_size', 2)
+            if not isinstance(b_size, int):
+                b_size = None
+        if k_size is not None:
+            _class.key_size = k_size
+        if b_size and b_size <= 1:
+            return
+        if 'mode' not in _class._argument_specification:
             return
         modes = extract_options(cipher, 'MODE_', 'SIV', 'OPENPGP')
         if not modes:
             raise RefineryCriticalException(F'No cipher block mode constants found in {cipher!r}')
-        cls._available_block_cipher_modes = OptionFactory(modes, ignorecase=True)
-        cls._argument_specification['mode'].merge_all(Arg(
+        _class._available_block_cipher_modes = OptionFactory(modes, ignorecase=True)
+        _class._argument_specification['mode'].merge_all(Arg(
             '-m', '--mode', type=str.upper, metavar='M', nargs=Arg.delete, choices=list(modes),
             help=(
                 'Choose cipher mode to be used. Possible values are: {}. By default, the CBC mode'
@@ -235,33 +225,48 @@ class StandardCipherExecutable(CipherExecutable):
 class StandardCipherUnit(CipherUnit, metaclass=StandardCipherExecutable):
 
     _available_block_cipher_modes: ClassVar[Type[Option]]
-    _cipher_object_factory: ClassVar[CipherObjectFactory]
+    _cipher_factory: ClassVar[CipherObjectFactory]
+    _cipher_interface: Optional[CipherInterface] = None
 
-    def _get_cipher_instance(self, **optionals) -> CipherInterface:
+    def _new_cipher(self, **optionals) -> CipherInterface:
         self.log_info(lambda: F'encryption key: {self.args.key.hex()}')
-        return self._cipher_object_factory.new(key=self.args.key, **optionals)
+        return self._cipher_factory.new(key=self.args.key, **optionals)
+
+    def _get_cipher(self, reset_cache=False) -> CipherInterface:
+        co = self._cipher_interface
+        if co is None or reset_cache:
+            self._cipher_interface = co = self._new_cipher()
+        return co
+
+    @property
+    def block_size(self) -> int:
+        return self._get_cipher().block_size
+
+    @property
+    def key_size(self) -> Optional[Sequence[int]]:
+        return self._get_cipher().key_size
 
     def encrypt(self, data: bytes) -> bytes:
-        return self._get_cipher_instance().encrypt(data)
+        cipher = self._get_cipher(True)
+        assert cipher.block_size == self.block_size
+        return cipher.encrypt(data)
 
     def decrypt(self, data: bytes) -> bytes:
-        cipher = self._get_cipher_instance()
+        cipher = self._get_cipher(True)
+        assert cipher.block_size == self.block_size
         try:
             return cipher.decrypt(data)
         except ValueError:
-            overlap = len(data) % self.blocksize
+            overlap = len(data) % self.block_size
             if not overlap:
                 raise
             data[-overlap:] = []
-            self.log_warn(F'removing {overlap} bytes from the input to make it a multiple of the {self.blocksize}-byte block size')
+            self.log_warn(F'removing {overlap} bytes from the input to make it a multiple of the {self.block_size}-byte block size')
             return cipher.decrypt(data)
 
 
 class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
 
-    blocksize: int
-    key_sizes: Tuple[int, ...]
-
     def __init__(
         self, key, iv=B'', padding=None, mode=None, raw=False,
         segment_size: Arg.Number('-S', '--segment-size',
@@ -282,20 +287,29 @@ class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
         )
 
     def _default_padding(self) -> Optional[str]:
-        padding = self.args.padding
+        padding = super()._default_padding()
         if padding is not None:
             return padding
-        if self.args.mode.name in {'ECB', 'CBC', 'PCBC'}:
-            return super()._default_padding()
-        return None
+        elif self.args.mode.name in {'ECB', 'CBC', 'PCBC'}:
+            return PADDINGS_LIB[0]
 
-    def _get_cipher_instance(self, **optionals) -> CipherInterface:
+    @property
+    def block_size(self) -> int:
+        provider = StandardCipherUnit
+        return provider.block_size.fget(self)
+
+    @property
+    def key_size(self) -> Sequence[int]:
+        provider = StandardCipherUnit
+        return provider.key_size.fget(self)
+
+    def _new_cipher(self, **optionals) -> CipherInterface:
         mode = self.args.mode.name
         if mode != 'ECB':
             iv = bytes(self.iv)
-            if mode == 'CTR' and len(iv) == self.blocksize:
+            if mode == 'CTR' and len(iv) == self.block_size:
                 from Crypto.Util import Counter
-                counter = Counter.new(self.blocksize * 8,
+                counter = Counter.new(self.block_size * 8,
                     initial_value=int.from_bytes(iv, 'big'))
                 optionals['counter'] = counter
             elif mode in ('CCM', 'EAX', 'GCM', 'SIV', 'OCB', 'CTR'):
@@ -310,9 +324,9 @@ class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
                     if al > 0:
                         optionals['assoc_len'] = al
                 bounds = {
-                    'CCM': (7, self.blocksize - 2),
-                    'OCB': (1, self.blocksize),
-                    'CTR': (1, self.blocksize),
+                    'CCM': (7, self.block_size - 2),
+                    'OCB': (1, self.block_size),
+                    'CTR': (1, self.block_size),
                 }.get(mode, None)
                 if bounds and len(iv) not in range(*bounds):
                     raise ValueError(F'Invalid nonce length, must be in {bounds} for {mode}.')
@@ -324,20 +338,21 @@ class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
                         raise ValueError(F'The given segment size {sz} is not a multiple of 8.')
                     if sz > 0:
                         optionals['segment_size'] = sz
-                if len(iv) > self.blocksize:
-                    self.log_warn(F'The IV has length {len(self.args.iv)} and will be truncated to the blocksize {self.blocksize}.')
-                    iv = iv[:self.blocksize]
-                elif len(iv) < self.blocksize:
-                    raise ValueError(F'The IV has length {len(self.args.iv)} but the block size is {self.blocksize}.')
+                if len(iv) > self.block_size:
+                    self.log_warn(F'The IV has length {len(self.args.iv)} and will be truncated to the block size {self.block_size}.')
+                    iv = iv[:self.block_size]
+                elif len(iv) < self.block_size:
+                    raise ValueError(F'The IV has length {len(self.args.iv)} but the block size is {self.block_size}.')
                 optionals['iv'] = iv
             self.log_info('initial vector:', iv.hex())
         if self.args.mode:
             optionals['mode'] = self.args.mode.value
-        return super()._get_cipher_instance(**optionals)
+        return super()._new_cipher(**optionals)
 
 
 class LatinCipherUnit(StreamCipherUnit, abstract=True):
-    key_sizes = 16, 32
+    key_size = {16, 32}
+    block_size = 1
 
     def __init__(
         self, key,
@@ -353,6 +368,6 @@ class LatinCipherStandardUnit(StandardCipherUnit):
     def __init__(self, key, nonce: Arg(help='The nonce. Default is the string {default}.') = B'REFINERY'):
         super().__init__(key, nonce=nonce)
 
-    def _get_cipher_instance(self, **optionals) -> Any:
+    def _new_cipher(self, **optionals) -> Any:
         self.log_info('one-time nonce:', self.args.nonce.hex())
-        return super()._get_cipher_instance(nonce=self.args.nonce)
+        return super()._new_cipher(nonce=self.args.nonce)
