@@ -3,9 +3,10 @@
 from Crypto.Cipher import CAST
 
 from refinery.units.crypto.cipher import StandardBlockCipherUnit
+from refinery.lib.crypto import PyCryptoFactoryWrapper
 
 
-class cast(StandardBlockCipherUnit, cipher=CAST):
+class cast(StandardBlockCipherUnit, cipher=PyCryptoFactoryWrapper(CAST)):
     """
     CAST encryption and decryption.
     """
