@@ -3,9 +3,10 @@
 from Crypto.Cipher import DES3
 
 from refinery.units.crypto.cipher import StandardBlockCipherUnit
+from refinery.lib.crypto import PyCryptoFactoryWrapper
 
 
-class des3(StandardBlockCipherUnit, cipher=DES3):
+class des3(StandardBlockCipherUnit, cipher=PyCryptoFactoryWrapper(DES3)):
     """
     3-DES encryption and decryption.
     """
