@@ -3,7 +3,7 @@
 """
 Implements various hashing algorithms.
 """
-from refinery.units import arg, Unit, abc
+from refinery.units import Arg, Unit, abc
 
 
 class HashUnit(Unit, abstract=True):
@@ -12,7 +12,7 @@ class HashUnit(Unit, abstract=True):
     def _algorithm(self, data: bytes):
         raise NotImplementedError
 
-    def __init__(self, text: arg('-t', help='Output a hexadecimal representation of the hash.') = False, **kwargs):
+    def __init__(self, text: Arg('-t', help='Output a hexadecimal representation of the hash.') = False, **kwargs):
         super().__init__(text=text, **kwargs)
 
     def process(self, data: bytes) -> bytes:
