@@ -3,7 +3,7 @@
 import string
 import itertools
 
-from refinery.units import Arg, Unit
+from refinery.units import Arg, Unit, Chunk
 
 from refinery.lib.meta import SizeInt, metavars
 from refinery.lib.structures import EOF, StructReader, StreamDetour
@@ -77,7 +77,7 @@ class struct(Unit):
         outputs = outputs or [F'{{{_SHARP}}}']
         super().__init__(spec=spec, outputs=outputs, until=until, count=count, multi=multi)
 
-    def process(self, data: bytearray):
+    def process(self, data: Chunk):
         formatter = string.Formatter()
         until = self.args.until
         until = until and PythonExpression(until, all_variables_allowed=True)
@@ -94,6 +94,8 @@ class struct(Unit):
                 spec = byteorder + spec
             return spec
 
+        previously_existing_variables = set(metavars(data).variable_names())
+
         it = itertools.count() if self.args.multi else (0,)
         for index in it:
 
@@ -102,8 +104,10 @@ class struct(Unit):
             if index >= self.args.count:
                 break
 
-            meta = metavars(data, ghost=True)
-            meta['index'] = index
+            meta = metavars(data)
+            meta.ghost = True
+            meta.update_index(index)
+
             args = []
             last = None
             checkpoint = reader.tell()
@@ -181,10 +185,13 @@ class struct(Unit):
                     used = set()
                     outputs.append(meta.format(template, self.codec, [full, *args], {_SHARP: last}, True, used=used))
                     for key in used:
-                        meta.pop(key, None)
+                        if key in previously_existing_variables:
+                            continue
+                        meta.discard(key)
 
                 for output in outputs:
-                    chunk = self.labelled(output, **meta)
+                    chunk = Chunk(output)
+                    chunk.meta.update(meta)
                     chunk.set_next_batch(index)
                     yield chunk
 
